var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { DataSource } from '@angular/cdk/collections';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Subject } from 'rxjs/Subject';
import { TableElement } from './table-element';
import { DefaultValidatorService } from './default-validator.service';
var TableDataSource = (function (_super) {
    __extends(TableDataSource, _super);
    function TableDataSource(data, dataType, validatorService) {
        var _this = _super.call(this) || this;
        _this.validatorService = validatorService;
        if (!validatorService)
            _this.validatorService = new DefaultValidatorService();
        if (dataType) {
            _this.dataConstructor = dataType;
        }
        else {
            if (data && data.length > 0)
                _this.dataKeys = Object.keys(data[0]);
            else
                throw new Error('You must define either a non empty array, or an associated class to build the table.');
        }
        _this.rowsSubject = new BehaviorSubject(_this.getRowsFromData(data));
        _this.datasourceSubject = new Subject();
        _this.rowsSubject.subscribe(function (rows) { return _this.updateDatasource(rows); });
        return _this;
    }
    TableDataSource.prototype.confirmEdit = function (row) {
        if (row.validator.valid) {
            var source = this.rowsSubject.getValue();
            source[row.id] = row;
            this.rowsSubject.next(source);
            row.editing = false;
            row.validator.disable();
        }
    };
    TableDataSource.prototype.confirmCreate = function (row) {
        if (row.validator.valid) {
            var source = this.rowsSubject.getValue();
            row.id = source.length - 1;
            this.rowsSubject.next(source);
            row.editing = false;
            row.validator.disable();
        }
    };
    TableDataSource.prototype.createNew = function () {
        var source = this.rowsSubject.getValue();
        if (source.length == 0 || source[source.length - 1].id > -1) {
            source.push(new TableElement({
                id: -1,
                editing: true,
                currentData: this.createNewObject(),
                source: this,
                validator: this.validatorService.getRowValidator(),
            }));
            this.rowsSubject.next(source);
        }
    };
    TableDataSource.prototype.delete = function (id) {
        var source = this.rowsSubject.getValue();
        var index = id == -1
            ? (source.length - 1)
            : id;
        source.splice(index, 1);
        this.rowsSubject.next(source);
    };
    TableDataSource.prototype.getDataFromRows = function (rows) {
        return rows
            .filter(function (row) { return row.id != -1; })
            .map(function (row) {
            return row.originalData ? row.originalData : row.currentData;
        });
    };
    TableDataSource.prototype.updateDatasource = function (rows) {
        this.datasourceSubject.next(this.getDataFromRows(rows));
    };
    TableDataSource.prototype.getRowsFromData = function (data) {
        var _this = this;
        return data.map(function (data, index) {
            var validator = _this.validatorService.getRowValidator();
            validator.disable();
            return new TableElement({
                id: index,
                editing: false,
                currentData: data,
                source: _this,
                validator: validator,
            });
        });
    };
    TableDataSource.prototype.createNewObject = function () {
        if (this.dataConstructor)
            return new this.dataConstructor();
        else {
            return this.dataKeys.reduce(function (obj, key) {
                obj[key] = undefined;
                return obj;
            }, {});
        }
    };
    /** Connect function called by the table to retrieve one stream containing the data to render. */
    TableDataSource.prototype.connect = function () {
        return this.rowsSubject.asObservable();
    };
    TableDataSource.prototype.disconnect = function () { };
    return TableDataSource;
}(DataSource));
export { TableDataSource };
//# sourceMappingURL=table-data-source.js.map